#include <TFT_eSPI.h>
#include <TFT_eWidget.h>
#include <FS.h>
#include "Adafruit_TinyUSB.h"

// TFT instance
TFT_eSPI tft = TFT_eSPI();

#define CALIBRATION_FILE "/TouchCalData1"
#define REPEAT_CAL true

void touch_calibrate();

void setup() {
  // Initialize Serial for debugging
  Serial.begin(115200);

  // Initialize the display
  tft.begin();
  tft.invertDisplay(1);
  tft.setRotation(0);  // Adjust the rotation if necessary
  tft.fillScreen(TFT_BLACK);

  // Calibrate the touch screen
  touch_calibrate();

  // Run the graphics test
  graphicsTest();
}

void loop() {
  tft.invertDisplay(1);
  delay(1000);
  tft.invertDisplay(0);
  delay(1000);
}

void touch_calibrate() {
  uint16_t calData[5];
  uint8_t calDataOK = 0;

  // Check file system exists
  if (!LittleFS.begin()) {
    Serial.println("Formating file system");
    LittleFS.format();
    LittleFS.begin();
  }

  // Check if calibration file exists and size is correct
  if (LittleFS.exists(CALIBRATION_FILE)) {
    if (REPEAT_CAL) {
      // Delete if we want to re-calibrate
      LittleFS.remove(CALIBRATION_FILE);
    } else {
      File f = LittleFS.open(CALIBRATION_FILE, "r");
      if (f) {
        if (f.readBytes((char *)calData, 14) == 14) {
          calDataOK = 1;
        }
        f.close();
      }
    }
  }

  if (calDataOK && !REPEAT_CAL) {
    // Calibration data valid
    tft.setTouch(calData);
  } else {
    // Data not valid so recalibrate
    tft.fillScreen(TFT_BLACK);
    tft.setCursor(20, 0);
    tft.setTextFont(2);
    tft.setTextSize(1);
    tft.setTextColor(TFT_WHITE, TFT_BLACK);

    tft.println("Touch corners as indicated");

    tft.setTextFont(1);
    tft.println();

    if (REPEAT_CAL) {
      tft.setTextColor(TFT_RED, TFT_BLACK);
      tft.println("Set REPEAT_CAL to false to stop this running again!");
    }

    tft.calibrateTouch(calData, TFT_MAGENTA, TFT_BLACK, 15);

    tft.setTextColor(TFT_GREEN, TFT_BLACK);
    tft.println("Calibration complete!");

    // Store data
    File f = LittleFS.open(CALIBRATION_FILE, "w");
    if (f) {
      f.write((const unsigned char *)calData, 14);
      f.close();
    }
  }
}

void graphicsTest() {
  // Clear the screen
  tft.fillScreen(TFT_BLACK);

  // Draw some lines
  tft.drawLine(0, 0, tft.width(), tft.height(), TFT_RED);
  tft.drawLine(tft.width(), 0, 0, tft.height(), TFT_RED);

  // Draw a rectangle
  tft.drawRect(10, 10, tft.width() - 20, tft.height() - 20, TFT_GREEN);

  // Draw a filled rectangle
  tft.fillRect(20, 20, tft.width() - 40, tft.height() - 40, TFT_BLUE);

  // Draw a circle
  tft.drawCircle(tft.width() / 2, tft.height() / 2, 50, TFT_YELLOW);

  // Draw a filled circle
  tft.fillCircle(tft.width() / 2, tft.height() / 2, 30, TFT_ORANGE);

  // Draw some text
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
  tft.setTextSize(2);
  tft.setCursor(10, tft.height() - 40);
  tft.print("Graphics Test");
}

// Generated by  : ImageConverter 565 v1.2
// Generated from: arduino.png
// Time generated: 24.07.2024 14:40:55
// Dimensions    : 128x128 pixels
// Size          : 32.768 Bytes

#include <avr/pgmspace.h>
int32_t ardicon_w = 128;
int32_t ardicon_h = 128;
prog_uint16_t arduino[0x4000] PROGMEM ={